<?php

namespace Drupal\synpay\Plugin\Synpay;

use Drupal\synpay\PluginManager\SynpayPluginBase;
use Drupal\synpay\PluginManager\SynpayPluginInterface;
use Ramsey\Uuid\Uuid;
// use GuzzleHttp\Client;
use GuzzleHttp\Exception\ClientException;
use Drupal\Component\Serialization\Json;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Provides Tinkoff Gateway.
 *
 * @SynpayAnnotation(
 *   id = "tinkoff",
 *   title = @Translation("Tinkoff"),
 * )
 */
class SynpayTinkoff extends SynpayPluginBase implements SynpayPluginInterface {
  const PAYMENT_EXPIRATION_PERIOD = '+1 hour';
  const LABEL = '"Тинькофф эквайринг"';
  const HOST = "https://securepay.tinkoff.ru/v2/";
  const HOST_TEST = "https://securepay.tinkoff.ru/v2/";
  const REGISTER = "rest/register.do";
  const DEPOSIT = "rest/deposit.do";
  const STATUS = "rest/getOrderStatusExtended.do";

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->plugin_id = $plugin_id;
    $this->order = isset($configuration['order']) ? $configuration['order'] : FALSE;
    $this->total = isset($configuration['total']) ? $configuration['total'] : FALSE;
    $this->host = \Drupal::request()->getSchemeAndHttpHost();
    $this->currency = 'RUB';
    $this->lang = \Drupal::languageManager()->getCurrentLanguage()->getId();
    if ($this->active == 1) {
      $mode = $this->config->get("{$plugin_id}_test_mode");
      switch ($mode) {
        case 0:
          $this->api = self::HOST;
          break;

        case 1:
          $this->api = self::HOST_TEST;
          break;
      }
      $this->client = \Drupal::httpClient();
      $this->options = [
        'headers' => [
          'Content-type' => 'application/x-www-form-urlencoded',
          'Cache-Control' => 'no-cache',
        ],
        'TerminalKey' => trim($this->user),
      ];
      // dsm($this->options);
    }
    else {
      return NULL;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function pay(string $order, int $total) {
    if ($order != $this->order) {
      \Drupal::messenger()->addError(t("Wrong order number"));
      return;
    }
    $order = Uuid::uuid4()->getHex()->toString();
    $order_amount = $total * 100;
    $params = $this->options;
    $params['Amount'] = $order_amount;
    $params['OrderId'] = $order;
    $params['SuccessURL'] = "{$this->host}/synpay/return_direct/$this->plugin_id";
    $params['FailURL'] = "{$this->host}/synpay/return_direct/$this->plugin_id?fail";
    dsm($params);
    try {
      $response = $this->client->post("{$this->api}Init", ['json' => $params]);
      dsm($response);
      $code = $response->getStatusCode();
      $result = Json::decode($response->getBody()->getContents());
      dsm($result);
    }
    catch (ClientException $exception) {
      $message = $exception->getMessage();
      \Drupal::messenger()->addError($message);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function registerOrder($order_id, $order_amount, $params) {
    if (is_null($this->client)) {
      return FALSE;
    }
    $data = $this->options;
    $order = Uuid::uuid4()->getHex()->toString();
    $data['OrderId'] = $order;
    $data['Amount'] = $order_amount;
    $data['SuccessURL'] = $params['returnUrl'];
    $data['FailURL'] = $params['failUrl'];
    $data['NotificationURL'] = "{$this->host}/payment/notify/$this->plugin_id";
    $data['Receipt']['Taxation'] = 'osn';
    if (!empty($params['orderBundle']['customerDetails']['email'])) {
      $data['Receipt']['Email'] = $params['orderBundle']['customerDetails']['email'];
    }
    foreach ($params['orderBundle']['cartItems']['items'] as $item) {
      $data['Receipt']['Items'][] = [
        'Name' => $item['name'],
        'Quantity' => $item['quantity']['value'],
        'Price' => $item['itemPrice'],
        'Amount' => $item['itemAmount'],
        'Tax' => 'vat20',
      ];
    }
    $url = "{$this->api}Init";
    // dsm($data);
    try {
      $answer = $this->client->request('POST', $url, ['json' => $data]);
      // dsm($answer);
      $code = $answer->getStatusCode();
      if ($code == 200) {
        $body = $answer->getBody()->getContents();
        $response = JSON::decode($body);
        // dsm($response);
        return [
          // 'data' => $data,
          'orderId' => $response['PaymentId'],
          'formUrl' => $response['PaymentURL'],
        ];
      }
    }
    catch (ActionException $exception) {
      $message = $exception->getMessage();
      \Drupal::messenger()->addError($message);
      \Drupal::messenger()->addWarning(t('Check login/password to your payment gateway!'));
    }
    return FALSE;
  }

  /**
   * Commerce Return.
   */
  public function onReturn(string $remote_id) {
    \Drupal::logger(__FUNCTION__ . __LINE__)->notice(
      '@j', ['@j' => json_encode($remote_id ?? '')]
    );
    $paymentInfoResponse = $this->client->getPaymentInfo($remote_id);
    switch ($paymentInfoResponse->status) {
      case PaymentStatus::SUCCEEDED:
        $result['status'] = 'completed';
        break;

      default:
        $result['status'] = 'authorization_voided';
    }
    return $result;
  }

  /**
   * Commerce Return.
   */
  public function onNotify(Request $request) {
    $paymentInfo = Json::decode(
      file_get_contents('php://input')
    );
    // \Drupal::logger(__FUNCTION__ . __LINE__)->notice(
    //   '@j', ['@j' => json_encode($paymentInfo ?? '')]
    // );
    // \Drupal::logger(__FUNCTION__ . __LINE__)->notice(
    //   '@j', ['@j' => json_encode($paymentInfo['PaymentId'] ?? '')]
    // );
    $payment_storage = \Drupal::entityTypeManager()->getStorage('commerce_payment');
    $payments = $payment_storage->loadByProperties(['remote_id' => $paymentInfo['PaymentId']]);
    $payment = reset($payments);
    if (!$payment) {
      return new Response('Bad request', 400);
    }
    $order = $payment->getOrder();
    if (!$order) {
      return new Response('Order not found', 404);
    }
    $time = \Drupal::time()->getRequestTime();
    if ($paymentInfo['Success']) {
      $order->state = 'completed';
      $order->setCompletedTime($time);
      $order->save();
      $payment->state = 'completed';
      $payment->setCompletedTime($time);
    }
    else {
      $payment->state = 'authorization_voided';
    }
    $payment->setRemoteState($paymentInfo['Status']);
    $payment->save();

    return new Response('OK', 200);
  }

  /**
   * Log.
   *
   * @param string $message
   *   Log message.
   */
  private function log($message) {
    \Drupal::logger('SynpayYandex')->info($message);
  }

}
